#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;
using System.Linq;
using System.Collections;
using HeuristicLab.GP.Interfaces;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.GP {
  public static class GeneralTreeSerializer {
    private static IFunctionTreeSerializer serializer;

    internal static string Export(IFunctionTree functionTree) {
      if (serializer != null || FindSerializer(functionTree)) {
        return serializer.Export(functionTree);
      } else throw new NotSupportedException("Can't serialize function trees. A matching export couldn't be found");
    }

    internal static IFunctionTree Import(string p) {
      if (serializer != null || FindSerializer(p)) {
        return serializer.Import(p);
      } else throw new NotSupportedException("Can't serialize function trees. A matching export couldn't be found");
    }

    private static bool FindSerializer(IFunctionTree functionTree) {
      DiscoveryService discoveryService = new DiscoveryService();
      string exported;

      foreach (IFunctionTreeSerializer exporter in discoveryService.GetInstances<IFunctionTreeSerializer>()) {
        if (exporter.TryExport(functionTree, out exported)) {
          GeneralTreeSerializer.serializer = exporter;
          return true;
        }
      }
      return false;
    }
    private static bool FindSerializer(string functionTree) {
      DiscoveryService discoveryService = new DiscoveryService();
      IFunctionTree exported;

      foreach (IFunctionTreeSerializer importer in discoveryService.GetInstances<IFunctionTreeSerializer>()) {
        if (importer.TryImport(functionTree, out exported)) {
          GeneralTreeSerializer.serializer = importer;
          return true;
        }
      }
      return false;
    }
  }
}
