#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.GP.Interfaces;
using System.Xml;

namespace HeuristicLab.GP {
  public class GeneticProgrammingModel : ItemBase, IGeneticProgrammingModel {
    private IFunctionTree functionTree;
    public IFunctionTree FunctionTree {
      get {
        return functionTree;
      }
      set {
        functionTree = value;
        Size = functionTree.GetSize();
        Height = functionTree.GetHeight();
      }
    }
    public int Size { get; set; }
    public int Height { get; set; }

    public GeneticProgrammingModel()
      : base() {
    }

    public GeneticProgrammingModel(IFunctionTree tree) : base() {
      FunctionTree = tree;
    }

    public override object Clone(IDictionary<Guid, object> clonedObjects) {
      GeneticProgrammingModel clone = (GeneticProgrammingModel)base.Clone(clonedObjects);
      clone.FunctionTree = FunctionTree.Clone();
      return clone;
    }

    public override XmlNode GetXmlNode(string name, XmlDocument document, IDictionary<Guid, IStorable> persistedObjects) {
      XmlNode node = base.GetXmlNode(name, document, persistedObjects);
      node.Value = GeneralTreeSerializer.Export(FunctionTree);
      return node;
    }

    public override void Populate(XmlNode node, IDictionary<Guid, IStorable> restoredObjects) {
      base.Populate(node, restoredObjects);
      FunctionTree = GeneralTreeSerializer.Import(node.Value);
    }

    public override IView CreateView() {
      return new FunctionTreeView(this.FunctionTree);
    }
  }
}
